import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py-db-knih",
    version="1.0.1",
    author="ROGR3",  
    author_email="your.email@example.com",  
    description="A Python library for scraping book information from databazeknih.cz",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ROGR3/py-db-knih",  # Replace with your GitHub URL
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Internet :: WWW/HTTP :: Browsers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing :: Markup :: HTML",
    ],
    python_requires=">=3.8",
    install_requires=[
        "requests>=2.32.0",
        "beautifulsoup4>=4.14.0",
        "lxml>=6.0.0",
    ],
    keywords="scraping, books, czech, databazeknih, web-scraping, library",
)